@@ -1,6 +1,6 @@
 package DBIx::Custom;
 
-our $VERSION = '0.1628';
+our $VERSION = '0.1629';
 
 use 5.008001;
 use strict;
@@ -20,21 +20,6 @@ __PACKAGE__->attr([qw/data_source dbh
                       dbi_options password user/]);
 
 __PACKAGE__->attr(cache => 1);
-__PACKAGE__->attr(cache_method => sub {
-    sub {
-        my $self = shift;
-        
-        $self->{_cached} ||= {};
-        
-        if (@_ > 1) {
-            $self->{_cached}{$_[0]} = $_[1] 
-        }
-        else {
-            return $self->{_cached}{$_[0]}
-        }
-    }
-});
-
 __PACKAGE__->attr(filters => sub {
     {
         encode_utf8 => sub { encode_utf8($_[0]) },
@@ -76,71 +61,72 @@ sub AUTOLOAD {
     return $self->$helper(@_);
 }
 
-sub auto_filter {
+sub apply_filter {
     my $self = shift;
     
-    $self->{auto_filter} ||= {};
+    $self->{filter} ||= {};
     
-    # Table
-    my $table = shift;
+    # Table and column informations
+    my ($table, @cs) = @_;
     
-    if (@_) {
-        # Column infomations
-        my @cs = @_;
+    if (@cs) {
         
         # Initialize filters
-        $self->{auto_filter}{bind} ||= {};
-        $self->{auto_filter}{fetch} ||= {};
+        $self->{filter}{out} ||= {};
+        $self->{filter}{in} ||= {};
         
-        # Create auto filters
-        foreach my $c (@cs) {
-            croak "Usage \$dbi->auto_filter(" .
-                  "TABLE, [COLUMN, BIND_FILTER, FETCH_FILTER], [...])"
-              unless ref $c eq 'ARRAY' && @$c == 3;
+        # Create filters
+        for (my $i = 0; $i < @cs; $i += 2) {
             
             # Column
-            my $column = $c->[0];
+            my $column = $cs[$i];
             
-            # Bind filter
-            my $bind_filter  = $c->[1];
-            if (ref $bind_filter eq 'CODE') {
-    	        $self->{auto_filter}{bind}{$table}{$column}
-    	          = $bind_filter;
-    	        $self->{auto_filter}{bind}{$table}{"$table.$column"}
-    	          = $bind_filter;
-            }
-            else {
-    	        croak qq{"$bind_filter" is not registered}
-    	          unless exists $self->filters->{$bind_filter};
-    	        
-    	        $self->{auto_filter}{bind}{$table}{$column}
-    	          = $self->filters->{$bind_filter};
-    	        $self->{auto_filter}{bind}{$table}{"$table.$column"}
-    	          = $self->filters->{$bind_filter};
-    	    }
+            # Filter
+            my $filter = $cs[$i + 1] || {};
+            my $in_filter = delete $filter->{in};
+            my $out_filter = delete $filter->{out};
+            croak "Usage \$dbi->apply_filter(" .
+                  "TABLE, COLUMN, {in => INFILTER, out => OUTFILTER}, ...)"
+              if ref $filter ne 'HASH' || keys %$filter;
             
-            # Fetch filter
-            my $fetch_filter = $c->[2];
-            if (ref $fetch_filter eq 'CODE') {
-    	        $self->{auto_filter}{fetch}{$table}{$column}
-    	          = $fetch_filter;
-    	        $self->{auto_filter}{fetch}{$table}{"$table.$column"}
-    	          = $fetch_filter;
+            # Out filter
+            if (ref $out_filter eq 'CODE') {
+                $self->{filter}{out}{$table}{$column}
+                  = $out_filter;
+                $self->{filter}{out}{$table}{"$table.$column"}
+                  = $out_filter;
             }
-            else {
-                croak qq{"$fetch_filter" is not registered}
-                  unless exists $self->filters->{$fetch_filter};
-                $self->{auto_filter}{fetch}{$table}{$column}
-                  = $self->filters->{$fetch_filter};
-                $self->{auto_filter}{fetch}{$table}{"$table.$column"}
-                  = $self->filters->{$fetch_filter};
+            elsif (defined $out_filter) {
+                croak qq{"$out_filter" is not registered}
+                  unless exists $self->filters->{$out_filter};
+                
+                $self->{filter}{out}{$table}{$column}
+                  = $self->filters->{$out_filter};
+                $self->{filter}{out}{$table}{"$table.$column"}
+                  = $self->filters->{$out_filter};
+            }
+            
+            # In filter
+            if (ref $in_filter eq 'CODE') {
+                $self->{filter}{in}{$table}{$column}
+                  = $in_filter;
+                $self->{filter}{in}{$table}{"$table.$column"}
+                  = $in_filter;
+            }
+            elsif (defined $in_filter) {
+                croak qq{"$in_filter" is not registered}
+                  unless exists $self->filters->{$in_filter};
+                $self->{filter}{in}{$table}{$column}
+                  = $self->filters->{$in_filter};
+                $self->{filter}{in}{$table}{"$table.$column"}
+                  = $self->filters->{$in_filter};
             }
         }
         
         return $self;
     }
     
-    return $self->{auto_filter};
+    return $self->{filter};
 }
 
 sub helper {
@@ -252,50 +238,8 @@ sub create_query {
     return $query;
 }
 
-sub default_bind_filter {
-    my $self = shift;
-    
-    if (@_) {
-        my $fname = $_[0];
-        
-        if (@_ && !$fname) {
-            $self->{default_bind_filter} = undef;
-        }
-        else {
-            croak qq{"$fname" is not registered}
-              unless exists $self->filters->{$fname};
-        
-            $self->{default_bind_filter} = $self->filters->{$fname};
-        }
-        return $self;
-    }
-    
-    return $self->{default_bind_filter};
-}
-
-sub default_fetch_filter {
-    my $self = shift;
-    my $fname = $_[0];
-    
-    if (@_) {
-        if (@_ && !$fname) {
-            $self->{default_fetch_filter} = undef;
-        }
-        else {
-            croak qq{"$fname" is not registered}
-              unless exists $self->filters->{$fname};
-        
-            $self->{default_fetch_filter} = $self->filters->{$fname};
-        }
-        
-        return $self;
-    }
-    
-    return $self->{default_fetch_filter}
-}
-
 our %VALID_DELETE_ARGS
-  = map { $_ => 1 } qw/auto_filter_table table where append filter allow_delete_all/;
+  = map { $_ => 1 } qw/table where append filter allow_delete_all/;
 
 sub delete {
     my ($self, %args) = @_;
@@ -313,11 +257,6 @@ sub delete {
     my $filter           = $args{filter};
     my $allow_delete_all = $args{allow_delete_all};
 
-    my $auto_filter_table = exists $args{auto_filter_table}
-                          ? $args{auto_filter_table}
-                          : [$table];
-    $auto_filter_table ||= [];    
-
     # Where keys
     my @where_keys = keys %$where;
     
@@ -341,7 +280,7 @@ sub delete {
     # Execute query
     my $ret_val = $self->execute(
         $source, param  => $where, filter => $filter,
-        auto_filter_table => $auto_filter_table);
+        table => $table);
     
     return $ret_val;
 }
@@ -350,7 +289,7 @@ sub delete_all { shift->delete(allow_delete_all => 1, @_) }
 
 sub DESTROY { }
 
-our %VALID_EXECUTE_ARGS = map { $_ => 1 } qw/param filter auto_filter_table/;
+our %VALID_EXECUTE_ARGS = map { $_ => 1 } qw/param filter table/;
 
 sub execute{
     my ($self, $query, %args)  = @_;
@@ -368,56 +307,61 @@ sub execute{
       unless ref $query;
     
     # Auto filter
-    my $auto_filter = {};
-    my $auto_filter_tables = $args{auto_filter_table} || [];
-    foreach my $table (@$auto_filter_tables) {
-        $auto_filter = {
-            %$auto_filter,
-            %{$self->{auto_filter}{bind}->{$table} || {}}
+    my $filter = {};
+    my $tables = $args{table} || [];
+    $tables = [$tables]
+      unless ref $tables eq 'ARRAY';
+    foreach my $table (@$tables) {
+        $filter = {
+            %$filter,
+            %{$self->{filter}{out}->{$table} || {}}
         }
     }
     
-    # Filter
-    my $filter = $args{filter} || $query->filter || {};
-    foreach my $column (keys %$filter) {
-        my $fname = $filter->{$column};
-        unless (ref $fname eq 'CODE') {
+    # Filter argument
+    my $f = $args{filter} || $query->filter || {};
+    foreach my $column (keys %$f) {
+        my $fname = $f->{$column};
+        if (!defined $fname) {
+            $f->{$column} = undef;
+        }
+        elsif (ref $fname ne 'CODE') {
           croak qq{"$fname" is not registered"}
             unless exists $self->filters->{$fname};
           
-          $filter->{$column} = $self->filters->{$fname};
+          $f->{$column} = $self->filters->{$fname};
         }
     }
-    $filter = {%$auto_filter, %$filter};
+    $filter = {%$filter, %$f};
     
-    # Create bind value
-    my $bind_values = $self->_build_bind_values($query, $params, $filter);
+    # Create bind values
+    my $binds = $self->_build_binds($params, $query->columns, $filter);
     
     # Execute
     my $sth      = $query->sth;
     my $affected;
-    eval {$affected = $sth->execute(@$bind_values)};
+    eval {$affected = $sth->execute(@$binds)};
     $self->_croak($@) if $@;
     
     # Return resultset if select statement is executed
     if ($sth->{NUM_OF_FIELDS}) {
         
-        # Auto fetch filter
-        my $auto_fetch_filter = {};
-	    foreach my $table (@$auto_filter_tables) {
-	        $auto_fetch_filter = {
-	            %$auto_filter,
-	            %{$self->{auto_filter}{fetch}{$table} || {}}
-	        }
-	    }
-	    
-		# Result
-		my $result = $self->result_class->new(
+        # Auto in filter
+        my $in_filter = {};
+        foreach my $table (@$tables) {
+            $in_filter = {
+                %$in_filter,
+                %{$self->{filter}{in}{$table} || {}}
+            }
+        }
+        
+        # Result
+        my $result = $self->result_class->new(
             sth            => $sth,
             filters        => $self->filters,
             filter_check   => $self->filter_check,
-            default_filter => $self->default_fetch_filter,
-            _auto_filter   => $auto_fetch_filter || {}
+            default_filter => $self->{default_in_filter},
+            filter         => $in_filter || {}
         );
 
         return $result;
@@ -441,7 +385,7 @@ sub expand {
 }
 
 our %VALID_INSERT_ARGS = map { $_ => 1 } qw/table param append
-                                            filter auto_filter_table/;
+                                            filter/;
 sub insert {
     my ($self, %args) = @_;
 
@@ -457,11 +401,6 @@ sub insert {
     my $append = $args{append} || '';
     my $filter = $args{filter};
     
-    my $auto_filter_table = exists $args{auto_filter_table}
-                          ? $args{auto_filter_table}
-                          : [$table];
-    $auto_filter_table ||= [];
-    
     # Insert keys
     my @insert_keys = keys %$param;
     
@@ -475,7 +414,7 @@ sub insert {
         $source,
         param  => $param,
         filter => $filter,
-        auto_filter_table => $auto_filter_table
+        table => $table
     );
     
     return $ret_val;
@@ -524,7 +463,7 @@ sub register_filter {
 }
 
 our %VALID_SELECT_ARGS
-  = map { $_ => 1 } qw/auto_filter_table table column where append relation filter/;
+  = map { $_ => 1 } qw/table column where append relation filter/;
 
 sub select {
     my ($self, %args) = @_;
@@ -543,10 +482,6 @@ sub select {
     my $relation = $args{relation};
     my $append   = $args{append};
     my $filter   = $args{filter};
-
-    my $auto_filter_table = exists $args{auto_filter_table}
-                          ? $args{auto_filter_table}
-                          : $tables;
     
     # Source of SQL
     my $source = 'select ';
@@ -602,7 +537,7 @@ sub select {
     # Execute query
     my $result = $self->execute(
         $source, param  => $param, filter => $filter,
-        auto_filter_table => $auto_filter_table);    
+        table => $tables);    
     
     return $result;
 }
@@ -643,7 +578,7 @@ sub txn_scope {
 }
 
 our %VALID_UPDATE_ARGS
-  = map { $_ => 1 } qw/auto_filter_table table param
+  = map { $_ => 1 } qw/table param
                        where append filter allow_update_all/;
 
 sub update {
@@ -663,11 +598,6 @@ sub update {
     my $filter           = $args{filter};
     my $allow_update_all = $args{allow_update_all};
     
-    my $auto_filter_table = exists $args{auto_filter_table}
-                          ? $args{auto_filter_table}
-                          : [$table];
-    $auto_filter_table ||= [];
-    
     # Update keys
     my @update_keys = keys %$param;
     
@@ -713,44 +643,38 @@ sub update {
     # Execute query
     my $ret_val = $self->execute($source, param  => $param, 
                                  filter => $filter,
-                                 auto_filter_table => $auto_filter_table);
+                                 table => $table);
     
     return $ret_val;
 }
 
 sub update_all { shift->update(allow_update_all => 1, @_) };
 
-sub _build_bind_values {
-    my ($self, $query, $params, $filter) = @_;
+sub _build_binds {
+    my ($self, $params, $columns, $filter) = @_;
     
-    # binding values
-    my @bind_values;
-
-    # Filter
-    $filter ||= {};
-    
-    # Parameter
-    $params ||= {};
+    # bind values
+    my @binds;
     
     # Build bind values
     my $count = {};
-    foreach my $column (@{$query->columns}) {
+    foreach my $column (@$columns) {
         
         # Value
         my $value = ref $params->{$column} eq 'ARRAY'
                   ? $params->{$column}->[$count->{$column} || 0]
                   : $params->{$column};
         
-        # Filtering
-        my $f = $filter->{$column} || $self->{default_bind_filter} || '';
+        # Filter
+        my $f = $filter->{$column} || $self->{default_out_filter} || '';
         
-        push @bind_values, $f ? $f->($value) : $value;
+        push @binds, $f ? $f->($value) : $value;
         
         # Count up 
         $count->{$column}++;
     }
     
-    return \@bind_values;
+    return \@binds;
 }
 
 sub _croak {
@@ -772,6 +696,64 @@ sub _croak {
     }
 }
 
+# Deprecated
+__PACKAGE__->attr(cache_method => sub {
+    sub {
+        my $self = shift;
+        
+        $self->{_cached} ||= {};
+        
+        if (@_ > 1) {
+            $self->{_cached}{$_[0]} = $_[1] 
+        }
+        else {
+            return $self->{_cached}{$_[0]}
+        }
+    }
+});
+
+sub default_bind_filter {
+    my $self = shift;
+    
+    if (@_) {
+        my $fname = $_[0];
+        
+        if (@_ && !$fname) {
+            $self->{default_out_filter} = undef;
+        }
+        else {
+            croak qq{"$fname" is not registered}
+              unless exists $self->filters->{$fname};
+        
+            $self->{default_out_filter} = $self->filters->{$fname};
+        }
+        return $self;
+    }
+    
+    return $self->{default_out_filter};
+}
+
+sub default_fetch_filter {
+    my $self = shift;
+    my $fname = $_[0];
+    
+    if (@_) {
+        if (@_ && !$fname) {
+            $self->{default_in_filter} = undef;
+        }
+        else {
+            croak qq{"$fname" is not registered}
+              unless exists $self->filters->{$fname};
+        
+            $self->{default_in_filter} = $self->filters->{$fname};
+        }
+        
+        return $self;
+    }
+    
+    return $self->{default_in_filter}
+}
+
 1;
 
 =head1 NAME
@@ -907,30 +889,6 @@ See L<DBIx::Custom::Guides> for more details.
 Enable parsed L<DBIx::Custom::Query> object caching.
 Default to 1.
 
-=head2 C<cache_method>
-
-    $dbi          = $dbi->cache_method(\&cache_method);
-    $cache_method = $dbi->cache_method
-
-Method to set and get caches.
-
-B<Example:>
-
-    $dbi->cache_method(
-        sub {
-            my $self = shift;
-            
-            $self->{_cached} ||= {};
-            
-            if (@_ > 1) {
-                $self->{_cached}{$_[0]} = $_[1] 
-            }
-            else {
-                return $self->{_cached}{$_[0]}
-            }
-        }
-    );
-
 =head2 C<data_source>
 
     my $data_source = $dbi->data_source;
@@ -1010,32 +968,28 @@ C<connect()> method use this value to connect the database.
 L<DBIx::Custom> inherits all methods from L<Object::Simple>
 and implements the following new ones.
 
-=head2 C<(experimental) auto_filter >
+=head2 C<(experimental) apply_filter >
 
-    $dbi->auto_filter(
+    $dbi->apply_filter(
         $table,
-        [$column1, $bind_filter1, $fetch_filter1],
-        [$column2, $bind_filter2, $fetch_filter2],
-        [...],
+        $column1 => {in => $infilter1, out => $outfilter1}
+        $column2 => {in => $infilter2, out => $outfilter2}
+        ...,
     );
 
-C<auto_filter> is automatically filter for columns of table.
+C<apply_filter> is automatically filter for columns of table.
 This have effect C<insert>, C<update>, C<delete>. C<select>
 and L<DBIx::Custom::Result> object. but this has'nt C<execute> method.
 
-If you want to have effect <execute< method, use C<auto_filter_table>
+If you want to have effect C<execute()> method, use C<table>
 arguments.
 
     $result = $dbi->execute(
         "select * from table1 where {= key1} and {= key2};",
          param => {key1 => 1, key2 => 2},
-         auto_filter_table => ['table1']
+         table => ['table1']
     );
     
-B<Example:>
-
-    $dbi->auto_filter('book', 'sale_date', 'to_date', 'date_to');
-
 =head2 C<begin_work>
 
     $dbi->begin_work;
@@ -1076,18 +1030,6 @@ instead of suger methods.
 
     $dbi->execute($query, {author => 'Ken', title => '%Perl%'});
 
-=head2 C<(deprecated) default_bind_filter>
-
-    my $default_bind_filter = $dbi->default_bind_filter;
-    $dbi                    = $dbi->default_bind_filter($fname);
-
-Default filter when parameter binding is executed.
-
-=head2 C<(deprecated) default_fetch_filter>
-
-    my $default_fetch_filter = $dbi->default_fetch_filter;
-    $dbi = $dbi->default_fetch_filter($fname);
-
 =head2 C<execute>
 
     my $result = $dbi->execute($query,  param => $params, filter => \%filter);
@@ -1376,7 +1318,7 @@ Rollback is automatically done.
 Note that this is feature of L<DBIx::TransactionManager>
 L<DBIx::TransactionManager> is required.
 
-=head2 C<table>
+=head2 C<(experimental) table>
 
     $dbi->table('book',
         insert => sub { ... },
@@ -1406,6 +1348,42 @@ B<Example:>
                      param  => {author => 'taro'},
                      filter => {author => 'encode_utf8'});
 
+=head2 C<(deprecated) default_bind_filter>
+
+    my $default_bind_filter = $dbi->default_bind_filter;
+    $dbi                    = $dbi->default_bind_filter($fname);
+
+Default filter when parameter binding is executed.
+
+=head2 C<(deprecated) default_fetch_filter>
+
+    my $default_fetch_filter = $dbi->default_fetch_filter;
+    $dbi = $dbi->default_fetch_filter($fname);
+
+=head2 C<(deprecated) cache_method>
+
+    $dbi          = $dbi->cache_method(\&cache_method);
+    $cache_method = $dbi->cache_method
+
+Method to set and get caches.
+
+B<Example:>
+
+    $dbi->cache_method(
+        sub {
+            my $self = shift;
+            
+            $self->{_cached} ||= {};
+            
+            if (@_ > 1) {
+                $self->{_cached}{$_[0]} = $_[1] 
+            }
+            else {
+                return $self->{_cached}{$_[0]}
+            }
+        }
+    );
+
 =head1 STABILITY
 
 L<DBIx::Custom> is now stable. APIs keep backword compatible in the feature.
